@@ -14,9 +14,6 @@
 /* All drivers initialising the SW cursor need this */
 #include "mipointer.h"
 
-/* All drivers implementing backing store need this */
-#include "mibstore.h"
-
 /* All drivers using the mi colormap manipulation need this */
 #include "micmap.h"
 
@@ -32,8 +29,10 @@
 
 #include "picturestr.h"
 
+#ifdef XvExtension
 #include "xf86xv.h"
 #include <X11/extensions/Xv.h>
+#endif
 
 /*
  * Driver data structures.
@@ -45,9 +44,9 @@
 #include <X11/Xproto.h>
 #include "scrnintstr.h"
 #include "servermd.h"
-#ifdef XFreeXDGA
+#ifdef USE_DGA
 #define _XF86DGA_SERVER_
-#include <X11/extensions/xf86dgastr.h>
+#include <X11/extensions/xf86dgaproto.h>
 #endif
 
 /* Mandatory functions */
@@ -55,21 +54,17 @@ static const OptionInfoRec *	DUMMYAvailableOptions(int chipid, int busid);
 static void     DUMMYIdentify(int flags);
 static Bool     DUMMYProbe(DriverPtr drv, int flags);
 static Bool     DUMMYPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool     DUMMYScreenInit(int Index, ScreenPtr pScreen, int argc,
-                                  char **argv);
-static Bool     DUMMYEnterVT(int scrnIndex, int flags);
-static void     DUMMYLeaveVT(int scrnIndex, int flags);
-static Bool     DUMMYCloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool     DUMMYScreenInit(SCREEN_INIT_ARGS_DECL);
+static Bool     DUMMYEnterVT(VT_FUNC_ARGS_DECL);
+static void     DUMMYLeaveVT(VT_FUNC_ARGS_DECL);
+static Bool     DUMMYCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool     DUMMYCreateWindow(WindowPtr pWin);
-static void     DUMMYFreeScreen(int scrnIndex, int flags);
-static ModeStatus DUMMYValidMode(int scrnIndex, DisplayModePtr mode,
+static void     DUMMYFreeScreen(FREE_SCREEN_ARGS_DECL);
+static ModeStatus DUMMYValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
                                  Bool verbose, int flags);
 static Bool	DUMMYSaveScreen(ScreenPtr pScreen, int mode);
 
 /* Internally used functions */
-static Bool     dummyModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
-static void	dummySave(ScrnInfoPtr pScrn);
-static void	dummyRestore(ScrnInfoPtr pScrn, Bool restoreText);
 static Bool	dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op,
 				pointer ptr);
 
@@ -85,6 +80,9 @@ static Bool	dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op,
 #define DUMMY_MINOR_VERSION PACKAGE_VERSION_MINOR
 #define DUMMY_PATCHLEVEL PACKAGE_VERSION_PATCHLEVEL
 
+#define DUMMY_MAX_WIDTH 32767
+#define DUMMY_MAX_HEIGHT 32767
+
 /*
  * This is intentionally screen-independent.  It indicates the binding
  * choice made in the first PreInit.
@@ -199,7 +197,7 @@ DUMMYFreeRec(ScrnInfoPtr pScrn)
 {
     if (pScrn->driverPrivate == NULL)
 	return;
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 }
 
@@ -280,7 +278,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
     ClockRangePtr clockRanges;
     int i;
     DUMMYPtr dPtr;
-    int maxClock = 230000;
+    int maxClock = 300000;
     GDevPtr device = xf86GetEntityInfo(pScrn->entityList[0])->device;
 
     if (flags & PROBE_DETECT) 
@@ -354,7 +352,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 
     xf86CollectOptions(pScrn, device->options);
     /* Process the options */
-    if (!(dPtr->Options = xalloc(sizeof(DUMMYOptions))))
+    if (!(dPtr->Options = malloc(sizeof(DUMMYOptions))))
 	return FALSE;
     memcpy(dPtr->Options, DUMMYOptions, sizeof(DUMMYOptions));
 
@@ -390,7 +388,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
     clockRanges->next = NULL;
     clockRanges->ClockMulFactor = 1;
     clockRanges->minClock = 11000;   /* guessed §§§ */
-    clockRanges->maxClock = 300000;
+    clockRanges->maxClock = maxClock;
     clockRanges->clockIndex = -1;		/* programmable */
     clockRanges->interlaceAllowed = TRUE; 
     clockRanges->doubleScanAllowed = TRUE;
@@ -402,8 +400,9 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 	int apertureSize = (pScrn->videoRam * 1024);
 	i = xf86ValidateModes(pScrn, pScrn->monitor->Modes,
 			      pScrn->display->modes, clockRanges,
-			      NULL, 256, 2048,(8 * pScrn->bitsPerPixel),
-			      128, 2048, pScrn->display->virtualX,
+			      NULL, 256, DUMMY_MAX_WIDTH,
+			      (8 * pScrn->bitsPerPixel),
+			      128, DUMMY_MAX_HEIGHT, pScrn->display->virtualX,
 			      pScrn->display->virtualY, apertureSize,
 			      LOOKUP_BEST_REFRESH);
 
@@ -457,25 +456,15 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 
 /* Mandatory */
 static Bool
-DUMMYEnterVT(int scrnIndex, int flags)
+DUMMYEnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-    
-    /* Should we re-save the text mode on each VT enter? */
-    if(!dummyModeInit(pScrn, pScrn->currentMode))
-      return FALSE;
-
-    DUMMYAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);    
-
     return TRUE;
 }
 
 /* Mandatory */
 static void
-DUMMYLeaveVT(int scrnIndex, int flags)
+DUMMYLeaveVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-    dummyRestore(pScrn, TRUE);
 }
 
 static void
@@ -515,7 +504,7 @@ static ScrnInfoPtr DUMMYScrn; /* static-globalize it */
 
 /* Mandatory */
 static Bool
-DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+DUMMYScreenInit(SCREEN_INIT_ARGS_DECL)
 {
     ScrnInfoPtr pScrn;
     DUMMYPtr dPtr;
@@ -526,22 +515,13 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
      * we need to get the ScrnInfoRec for this screen, so let's allocate
      * one first thing
      */
-    pScrn = xf86Screens[pScreen->myNum];
+    pScrn = xf86ScreenToScrn(pScreen);
     dPtr = DUMMYPTR(pScrn);
     DUMMYScrn = pScrn;
 
 
-    if (!(dPtr->FBBase = xalloc(pScrn->videoRam * 1024)))
+    if (!(dPtr->FBBase = malloc(pScrn->videoRam * 1024)))
 	return FALSE;
-    
-    /*
-     * next we save the current state and setup the first mode
-     */
-    dummySave(pScrn);
-    
-    if (!dummyModeInit(pScrn,pScrn->currentMode))
-	return FALSE;
-    DUMMYAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
 
     /*
      * Reset visual list.
@@ -588,10 +568,12 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     xf86SetBlackWhitePixels(pScreen);
 
+#ifdef USE_DGA
     DUMMYDGAInit(pScreen);
+#endif
     
     if (dPtr->swCursor)
-	xf86DrvMsg(scrnIndex, X_CONFIG, "Using Software Cursor.\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Using Software Cursor.\n");
 
     {
 
@@ -610,7 +592,6 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 		   , lines - pScrn->virtualY);
     }
 
-    miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
     xf86SetSilkenMouse(pScreen);
 	
@@ -621,7 +602,7 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!dPtr->swCursor) {
       /* HW cursor functions */
       if (!DUMMYCursorInit(pScreen)) {
-	  xf86DrvMsg(scrnIndex, X_ERROR,
+	  xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		     "Hardware cursor initialization failed\n");
 	  return FALSE;
       }
@@ -660,48 +641,26 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
 /* Mandatory */
 Bool
-DUMMYSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+DUMMYSwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-    return dummyModeInit(xf86Screens[scrnIndex], mode);
+    return TRUE;
 }
 
 /* Mandatory */
 void
-DUMMYAdjustFrame(int scrnIndex, int x, int y, int flags)
+DUMMYAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn;
-    int Base; 
-
-    pScrn = xf86Screens[scrnIndex];
-
-    Base = (y * pScrn->displayWidth + x) >> 2;
-
-    /* Scale Base by the number of bytes per pixel. */
-    switch (pScrn->depth) {
-    case  8 :
-	break;
-    case 15 :
-    case 16 :
-	Base *= 2;
-	break;
-    case 24 :
-	Base *= 3;
-	break;
-    default :
-	break;
-    }
 }
 
 /* Mandatory */
 static Bool
-DUMMYCloseScreen(int scrnIndex, ScreenPtr pScreen)
+DUMMYCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     DUMMYPtr dPtr = DUMMYPTR(pScrn);
 
     if(pScrn->vtSema){
- 	dummyRestore(pScrn, TRUE);
-	xfree(dPtr->FBBase);
+	free(dPtr->FBBase);
     }
 
     if (dPtr->CursorInfo)
@@ -709,14 +668,15 @@ DUMMYCloseScreen(int scrnIndex, ScreenPtr pScreen)
 
     pScrn->vtSema = FALSE;
     pScreen->CloseScreen = dPtr->CloseScreen;
-    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+    return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 /* Optional */
 static void
-DUMMYFreeScreen(int scrnIndex, int flags)
+DUMMYFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    DUMMYFreeRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    DUMMYFreeRec(pScrn);
 }
 
 static Bool
@@ -726,7 +686,7 @@ DUMMYSaveScreen(ScreenPtr pScreen, int mode)
     DUMMYPtr dPtr;
 
     if (pScreen != NULL) {
-	pScrn = xf86Screens[pScreen->myNum];
+	pScrn = xf86ScreenToScrn(pScreen);
 	dPtr = DUMMYPTR(pScrn);
 
 	dPtr->screenSaver = xf86IsUnblank(mode);
@@ -736,53 +696,42 @@ DUMMYSaveScreen(ScreenPtr pScreen, int mode)
 
 /* Optional */
 static ModeStatus
-DUMMYValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+DUMMYValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
     return(MODE_OK);
 }
 
-static void
-dummySave(ScrnInfoPtr pScrn)
-{
-}
-
-static void 
-dummyRestore(ScrnInfoPtr pScrn, Bool restoreText)
-{
-}
-    
-static Bool
-dummyModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
-{
-    dummyRestore(pScrn, FALSE);
-    
-    return(TRUE);
-}
-
-
 Atom VFB_PROP  = 0;
 #define  VFB_PROP_NAME  "VFB_IDENT"
 
-
-
 static Bool
 DUMMYCreateWindow(WindowPtr pWin)
 {
+    ScreenPtr pScreen = pWin->drawable.pScreen;
     DUMMYPtr dPtr = DUMMYPTR(DUMMYScrn);
     WindowPtr pWinRoot;
     int ret;
-	
-    ret = dPtr->CreateWindow(pWin);
+
+    pScreen->CreateWindow = dPtr->CreateWindow;
+    ret = pScreen->CreateWindow(pWin);
+    dPtr->CreateWindow = pScreen->CreateWindow;
+    pScreen->CreateWindow = DUMMYCreateWindow;
+
     if(ret != TRUE)
 	return(ret);
 	
     if(dPtr->prop == FALSE) {
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 8
         pWinRoot = WindowTable[DUMMYScrn->pScreen->myNum];
+#else
+        pWinRoot = DUMMYScrn->pScreen->root;
+#endif
         if (! ValidAtom(VFB_PROP))
             VFB_PROP = MakeAtom(VFB_PROP_NAME, strlen(VFB_PROP_NAME), 1);
 
-        ret = ChangeWindowProperty(pWinRoot, VFB_PROP, XA_STRING, 
-		8, PropModeReplace, (int)4, (pointer)"TRUE", FALSE);
+        ret = dixChangeWindowProperty(serverClient, pWinRoot, VFB_PROP,
+                                      XA_STRING, 8, PropModeReplace,
+                                      (int)4, (pointer)"TRUE", FALSE);
 	if( ret != Success)
 		ErrorF("Could not set VFB root window property");
         dPtr->prop = TRUE;
@@ -792,6 +741,9 @@ DUMMYCreateWindow(WindowPtr pWin)
     return TRUE;
 }
 
+#ifndef HW_SKIP_CONSOLE
+#define HW_SKIP_CONSOLE 4
+#endif
 
 static Bool
 dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op, pointer ptr)
@@ -801,7 +753,7 @@ dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op, pointer ptr)
     switch (op) {
 	case GET_REQUIRED_HW_INTERFACES:
 	    flag = (CARD32*)ptr;
-	    (*flag) = 0;
+	    (*flag) = HW_SKIP_CONSOLE;
 	    return TRUE;
 	default:
 	    return FALSE;
